==> 2.2.0-beta.2.tar.gz: lib/src/widgets/story_block/story_block_platform_view.dart <==
import 'dart:async';

import 'package:flutter/foundation.dart';
import 'package:flutter/gestures.dart';
import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';
import 'package:flutter/services.dart';
import 'package:fw_flutter_sdk/src/utils/fw_logger_util.dart';
import 'package:visibility_detector/visibility_detector.dart';

import '../../models/ad_configuration.dart';
import '../../models/fw_error.dart';
import '../../models/story_block_native_configuration.dart';
import '../../models/story_block_source.dart';
import '../../utils/model_utils/ad_configuration_util.dart';
import '../../utils/model_utils/story_block_configuration_util.dart';
import '../mixins/android_platform_view_mixin.dart';
import 'story_block_platform_handler.dart';

typedef StoryBlockPlatformViewLoadFinishedCallback = void Function(
    FWError? error);
typedef StoryBlockPlatformViewCreatedCallback = void Function(
    StoryBlockPlatformHandler handler);
typedef StoryBlockPlatformViewFullScreenStateChanged = void Function(
    bool isFullScreen);

class StoryBlockPlatformView extends StatefulWidget {
  final StoryBlockSource source;
  final String? channel;
  final String? playlist;
  final Map<String, List<String>>? dynamicContentParameters;
  final String? hashtagFilterExpression;
  final List<String>? productIds;
  final String? contentId;
  final double? cornerRadius;
  final AdConfiguration? adConfiguration;
  final StoryBlockNativeConfiguration? storyBlockConfiguration;
  final StoryBlockPlatformViewCreatedCallback? onStoryBlockformViewCreated;
  final StoryBlockPlatformViewLoadFinishedCallback?
      onStoryBlockPlatformViewLoadFinished;
  final StoryBlockPlatformViewFullScreenStateChanged?
      onStoryBlockPlatformViewFullScreenStateChanged;
  const StoryBlockPlatformView({
    Key? key,
    required this.source,
    this.channel,
    this.playlist,
    this.dynamicContentParameters,
    this.hashtagFilterExpression,
    this.productIds,
    this.contentId,
    this.cornerRadius,
    this.adConfiguration,
    this.storyBlockConfiguration,
    this.onStoryBlockformViewCreated,
    this.onStoryBlockPlatformViewLoadFinished,
    this.onStoryBlockPlatformViewFullScreenStateChanged,
  }) : super(key: key);

  @override
  State<StoryBlockPlatformView> createState() => _StoryBlockPlatformViewState();
}

class _StoryBlockPlatformViewState extends State<StoryBlockPlatformView>
    with AndroidPlatformViewMixin {
  final String _viewType = "fw.fondor.view/story_block";
  Completer<StoryBlockPlatformHandler> _handlerCompleter = Completer();
  final _iOSPlatformViewKey = GlobalKey();

  @override
  void initState() {
    super.initState();
  }

  @override
  void didUpdateWidget(covariant StoryBlockPlatformView oldWidget) {
    super.didUpdateWidget(oldWidget);

    _handlerCompleter.future.then((handler) {
      handler.updateWidget(widget);
    });
  }

  @override
  void dispose() {
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    if (defaultTargetPlatform == TargetPlatform.android) {
      return VisibilityDetector(
        key: const Key("firework-story-block-platform-view"),
        child: buildAndroidPlatformView(_buildAndroidPlatformView),
        onVisibilityChanged: (visibilityInfo) {
          FWLoggerUtil.log(
              "firework-story-block-platform-view visibilityInfo.visibleFraction: ${visibilityInfo.visibleFraction}");
          _handlerCompleter.future.then((handler) {
            if (visibilityInfo.visibleFraction == 0) {
              handler.hide();
            } else {
              handler.show();
            }
          });
        },
      );
    } else {
      final cornerRadius = widget.cornerRadius;
      return Container(
        decoration: BoxDecoration(
          borderRadius: BorderRadius.circular(cornerRadius ?? 0),
        ),
        clipBehavior: Clip.hardEdge,
        child: _buildIOSPlatformView(context),
      );
    }
  }

  Widget _buildAndroidPlatformView(BuildContext context, ValueKey key) {
    final creationParams = _generateCreationParams();
    const layoutDirection = TextDirection.ltr;
    const creationParamsCodec = StandardMessageCodec();
    return PlatformViewLink(
      key: ValueKey(key),
      viewType: _viewType,
      surfaceFactory:
          (BuildContext context, PlatformViewController controller) {
        return AndroidViewSurface(
          controller: controller as AndroidViewController,
          gestureRecognizers: _defaultGestureRecognizers(),
          hitTestBehavior: PlatformViewHitTestBehavior.opaque,
        );
      },
      onCreatePlatformView: (PlatformViewCreationParams params) {
        return PlatformViewsService.initExpensiveAndroidView(
          id: params.id,
          viewType: _viewType,
          layoutDirection: layoutDirection,
          creationParams: creationParams,
          creationParamsCodec: creationParamsCodec,
          onFocus: () {
            params.onFocusChanged(true);
          },
        )
          ..addOnPlatformViewCreatedListener(params.onPlatformViewCreated)
          ..addOnPlatformViewCreatedListener(_onPlatformViewCreated)
          ..create();
      },
    );
  }

  Widget _buildIOSPlatformView(BuildContext context) {
    Map<String, dynamic> creationParams = _generateCreationParams();
    const layoutDirection = TextDirection.ltr;

    return UiKitView(
      key: _iOSPlatformViewKey,
      viewType: _viewType,
      onPlatformViewCreated: _onPlatformViewCreated,
      layoutDirection: layoutDirection,
      creationParams: creationParams,
      creationParamsCodec: const StandardMessageCodec(),
      gestureRecognizers: _defaultGestureRecognizers(),
    );
  }

  Map<String, dynamic> _generateCreationParams() {
    Map<String, dynamic> creationParams = <String, dynamic>{
      "source": widget.source.name,
    };

    if (widget.channel != null) {
      creationParams["channel"] = widget.channel!;
    }

    if (widget.playlist != null) {
      creationParams["playlist"] = widget.playlist!;
    }

    if (widget.dynamicContentParameters != null) {
      creationParams["dynamicContentParameters"] =
          widget.dynamicContentParameters!;
    }

    if (widget.hashtagFilterExpression != null) {
      creationParams["hashtagFilterExpression"] =
          widget.hashtagFilterExpression!;
    }

    if (widget.productIds != null) {
      creationParams["productIds"] = widget.productIds!;
    }

    if (widget.contentId != null) {
      creationParams["contentId"] = widget.contentId!;
    }

    if (defaultTargetPlatform == TargetPlatform.android) {
      if (widget.cornerRadius != null) {
        creationParams["cornerRadius"] = widget.cornerRadius!;
      }
    }

    if (widget.adConfiguration != null) {
      creationParams["adConfiguration"] =
          AdConfigurationUtil.toJson(widget.adConfiguration!);
    }

    if (widget.storyBlockConfiguration != null) {
      creationParams["storyBlockConfiguration"] =
          StoryBlockConfigurationUtil.toJson(widget.storyBlockConfiguration);
    }

    if (defaultTargetPlatform == TargetPlatform.android) {
      creationParams["marginRight"] = androidMarginRight;
    }

    return creationParams;
  }

  Set<Factory<HorizontalDragGestureRecognizer>> _defaultGestureRecognizers() {
    return <Factory<HorizontalDragGestureRecognizer>>{
      Factory<HorizontalDragGestureRecognizer>(
        () => HorizontalDragGestureRecognizer(),
      ),
    };
  }

  void _onPlatformViewCreated(int viewId) {
    final handler = StoryBlockPlatformHandler(viewId: viewId, widget: widget);
    if (_handlerCompleter.isCompleted) {
      _handlerCompleter = Completer();
    }
    _handlerCompleter.complete(handler);

    widget.onStoryBlockformViewCreated?.call(handler);

    WidgetsBinding.instance.addPostFrameCallback((_) {
      if (defaultTargetPlatform == TargetPlatform.iOS) {
        if (_iOSPlatformViewKey.currentContext != null) {
          final renderObject =
              _iOSPlatformViewKey.currentContext!.findRenderObject();
          FWLoggerUtil.log('StoryBlockPlatformView renderObject $renderObject');
          if (renderObject != null) {
            final renderBox = _findRenderUiKitView(renderObject);
            FWLoggerUtil.log('StoryBlockPlatformView renderBox $renderBox');
            if (renderBox != null) {
              handler.updateRenderUiKitView(renderBox);
            }
          }
        }
      }
    });
  }

  RenderUiKitView? _findRenderUiKitView(RenderObject object) {
    if (object is RenderUiKitView) {
      return object;
    }
    RenderUiKitView? result;
    FWLoggerUtil.log("StoryBlockPlatformView before visitChildren");
    object.visitChildren((child) {
      FWLoggerUtil.log("StoryBlockPlatformView in visitChildren $child");

      final childResult = _findRenderUiKitView(child);
      if (childResult != null) {
        result = childResult;
      }
    });
    FWLoggerUtil.log("StoryBlockPlatformView after visitChildren");
    return result;
  }
}

==> 2.2.1-beta.2.tar.gz: lib/src/widgets/story_block/story_block_platform_view.dart <==
import 'dart:async';

import 'package:flutter/foundation.dart';
import 'package:flutter/gestures.dart';
import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';
import 'package:flutter/services.dart';
import 'package:fw_flutter_sdk/src/utils/fw_logger_util.dart';
import 'package:visibility_detector/visibility_detector.dart';

import '../../models/ad_configuration.dart';
import '../../models/fw_error.dart';
import '../../models/story_block_native_configuration.dart';
import '../../models/story_block_source.dart';
import '../../utils/model_utils/ad_configuration_util.dart';
import '../../utils/model_utils/story_block_configuration_util.dart';
import '../mixins/android_platform_view_mixin.dart';
import 'story_block_platform_handler.dart';

typedef StoryBlockPlatformViewLoadFinishedCallback = void Function(
    FWError? error);
typedef StoryBlockPlatformViewCreatedCallback = void Function(
    StoryBlockPlatformHandler handler);
typedef StoryBlockPlatformViewFullScreenStateChanged = void Function(
    bool isFullScreen);

class StoryBlockPlatformView extends StatefulWidget {
  final StoryBlockSource source;
  final String? channel;
  final String? playlist;
  final Map<String, List<String>>? dynamicContentParameters;
  final String? hashtagFilterExpression;
  final List<String>? productIds;
  final String? contentId;
  final double? cornerRadius;
  final AdConfiguration? adConfiguration;
  final StoryBlockNativeConfiguration? storyBlockConfiguration;
  final StoryBlockPlatformViewCreatedCallback? onStoryBlockformViewCreated;
  final StoryBlockPlatformViewLoadFinishedCallback?
      onStoryBlockPlatformViewLoadFinished;
  final StoryBlockPlatformViewFullScreenStateChanged?
      onStoryBlockPlatformViewFullScreenStateChanged;
  const StoryBlockPlatformView({
    Key? key,
    required this.source,
    this.channel,
    this.playlist,
    this.dynamicContentParameters,
    this.hashtagFilterExpression,
    this.productIds,
    this.contentId,
    this.cornerRadius,
    this.adConfiguration,
    this.storyBlockConfiguration,
    this.onStoryBlockformViewCreated,
    this.onStoryBlockPlatformViewLoadFinished,
    this.onStoryBlockPlatformViewFullScreenStateChanged,
  }) : super(key: key);

  @override
  State<StoryBlockPlatformView> createState() => _StoryBlockPlatformViewState();
}

class _StoryBlockPlatformViewState extends State<StoryBlockPlatformView>
    with AndroidPlatformViewMixin {
  final String _viewType = "fw.fondor.view/story_block";
  Completer<StoryBlockPlatformHandler> _handlerCompleter = Completer();
  final _iOSPlatformViewKey = GlobalKey();
  bool _androidPlatformViewVisible = false;

  @override
  void initState() {
    super.initState();
  }

  @override
  void didUpdateWidget(covariant StoryBlockPlatformView oldWidget) {
    super.didUpdateWidget(oldWidget);

    _handlerCompleter.future.then((handler) {
      handler.updateWidget(widget);
    });
  }

  @override
  void dispose() {
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    if (defaultTargetPlatform == TargetPlatform.android) {
      return VisibilityDetector(
        key: const Key("firework-story-block-platform-view"),
        child: GestureDetector(
          onTap: () {
            FWLoggerUtil.log("Android PlatformView GestureDetector onTap");
          },
          child: AbsorbPointer(
            absorbing: !_androidPlatformViewVisible,
            child: buildAndroidPlatformView(_buildAndroidPlatformView),
          ),
        ),
        onVisibilityChanged: (visibilityInfo) {
          FWLoggerUtil.log(
              "firework-story-block-platform-view visibilityInfo.visibleFraction: ${visibilityInfo.visibleFraction}");
          _handlerCompleter.future.then((handler) {
            if (visibilityInfo.visibleFraction == 0) {
              handler.hide();
            } else {
              handler.show();
            }
          });
          FWLoggerUtil.log(
              "Android PlatformView onVisibilityChanged ${visibilityInfo.visibleFraction}");
          setState(() {
            _androidPlatformViewVisible = visibilityInfo.visibleFraction > 0;
          });
        },
      );
    } else {
      final cornerRadius = widget.cornerRadius;
      return Container(
        decoration: BoxDecoration(
          borderRadius: BorderRadius.circular(cornerRadius ?? 0),
        ),
        clipBehavior: Clip.hardEdge,
        child: _buildIOSPlatformView(context),
      );
    }
  }

  Widget _buildAndroidPlatformView(BuildContext context, ValueKey key) {
    final creationParams = _generateCreationParams();
    const layoutDirection = TextDirection.ltr;
    const creationParamsCodec = StandardMessageCodec();
    return PlatformViewLink(
      key: ValueKey(key),
      viewType: _viewType,
      surfaceFactory:
          (BuildContext context, PlatformViewController controller) {
        return AndroidViewSurface(
          controller: controller as AndroidViewController,
          gestureRecognizers: _defaultGestureRecognizersAndroid(),
          hitTestBehavior: PlatformViewHitTestBehavior.opaque,
        );
      },
      onCreatePlatformView: (PlatformViewCreationParams params) {
        return PlatformViewsService.initExpensiveAndroidView(
          id: params.id,
          viewType: _viewType,
          layoutDirection: layoutDirection,
          creationParams: creationParams,
          creationParamsCodec: creationParamsCodec,
          onFocus: () {
            params.onFocusChanged(true);
          },
        )
          ..addOnPlatformViewCreatedListener(params.onPlatformViewCreated)
          ..addOnPlatformViewCreatedListener(_onPlatformViewCreated)
          ..create();
      },
    );
  }

  Widget _buildIOSPlatformView(BuildContext context) {
    Map<String, dynamic> creationParams = _generateCreationParams();
    const layoutDirection = TextDirection.ltr;

    return UiKitView(
      key: _iOSPlatformViewKey,
      viewType: _viewType,
      onPlatformViewCreated: _onPlatformViewCreated,
      layoutDirection: layoutDirection,
      creationParams: creationParams,
      creationParamsCodec: const StandardMessageCodec(),
      gestureRecognizers: _defaultGestureRecognizersIOS(),
    );
  }

  Map<String, dynamic> _generateCreationParams() {
    Map<String, dynamic> creationParams = <String, dynamic>{
      "source": widget.source.name,
    };

    if (widget.channel != null) {
      creationParams["channel"] = widget.channel!;
    }

    if (widget.playlist != null) {
      creationParams["playlist"] = widget.playlist!;
    }

    if (widget.dynamicContentParameters != null) {
      creationParams["dynamicContentParameters"] =
          widget.dynamicContentParameters!;
    }

    if (widget.hashtagFilterExpression != null) {
      creationParams["hashtagFilterExpression"] =
          widget.hashtagFilterExpression!;
    }

    if (widget.productIds != null) {
      creationParams["productIds"] = widget.productIds!;
    }

    if (widget.contentId != null) {
      creationParams["contentId"] = widget.contentId!;
    }

    if (defaultTargetPlatform == TargetPlatform.android) {
      if (widget.cornerRadius != null) {
        creationParams["cornerRadius"] = widget.cornerRadius!;
      }
    }

    if (widget.adConfiguration != null) {
      creationParams["adConfiguration"] =
          AdConfigurationUtil.toJson(widget.adConfiguration!);
    }

    if (widget.storyBlockConfiguration != null) {
      creationParams["storyBlockConfiguration"] =
          StoryBlockConfigurationUtil.toJson(widget.storyBlockConfiguration);
    }

    if (defaultTargetPlatform == TargetPlatform.android) {
      creationParams["marginRight"] = androidMarginRight;
    }

    return creationParams;
  }

  Set<Factory<HorizontalDragGestureRecognizer>>
      _defaultGestureRecognizersIOS() {
    return <Factory<HorizontalDragGestureRecognizer>>{
      Factory<HorizontalDragGestureRecognizer>(
        () => HorizontalDragGestureRecognizer(),
      ),
    };
  }

  Set<Factory<OneSequenceGestureRecognizer>>
      _defaultGestureRecognizersAndroid() {
    return <Factory<OneSequenceGestureRecognizer>>{};
  }

  void _onPlatformViewCreated(int viewId) {
    final handler = StoryBlockPlatformHandler(viewId: viewId, widget: widget);
    if (_handlerCompleter.isCompleted) {
      _handlerCompleter = Completer();
    }
    _handlerCompleter.complete(handler);

    widget.onStoryBlockformViewCreated?.call(handler);

    WidgetsBinding.instance.addPostFrameCallback((_) {
      if (defaultTargetPlatform == TargetPlatform.iOS) {
        if (_iOSPlatformViewKey.currentContext != null) {
          final renderObject =
              _iOSPlatformViewKey.currentContext!.findRenderObject();
          FWLoggerUtil.log('StoryBlockPlatformView renderObject $renderObject');
          if (renderObject != null) {
            final renderBox = _findRenderUiKitView(renderObject);
            FWLoggerUtil.log('StoryBlockPlatformView renderBox $renderBox');
            if (renderBox != null) {
              handler.updateRenderUiKitView(renderBox);
            }
          }
        }
      }
    });
  }

  RenderUiKitView? _findRenderUiKitView(RenderObject object) {
    if (object is RenderUiKitView) {
      return object;
    }
    RenderUiKitView? result;
    FWLoggerUtil.log("StoryBlockPlatformView before visitChildren");
    object.visitChildren((child) {
      FWLoggerUtil.log("StoryBlockPlatformView in visitChildren $child");

      final childResult = _findRenderUiKitView(child);
      if (childResult != null) {
        result = childResult;
      }
    });
    FWLoggerUtil.log("StoryBlockPlatformView after visitChildren");
    return result;
  }
}
